@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//                Copyright (c) 1998 - 2016 David Bryant.                 //
+//                Copyright (c) 1998 - 2019 David Bryant.                 //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -29,7 +29,7 @@
 
 static const char *sign_on = "\n"
 " WVTEST  libwavpack Tester/Exerciser for WavPack  %s Version %s\n"
-" Copyright (c) 2016 David Bryant.  All Rights Reserved.\n\n";
+" Copyright (c) 2019 David Bryant.  All Rights Reserved.\n\n";
 
 static const char *version_warning = "\n"
 " WARNING: WVTEST using libwavpack version %s, expected %s (see README)\n\n";
@@ -342,7 +342,7 @@ static int seeking_test (char *filename, uint32_t test_count)
         }
 
         sample_count = chunk_count = 0;
-        MD5Init (&md5_global);
+        MD5_Init (&md5_global);
 
         // read the entire file, calculating the MD5 sums for the whole file and for each "chunk"
 
@@ -353,11 +353,11 @@ static int seeking_test (char *filename, uint32_t test_count)
                 break;
 
             store_samples (decoded_samples, decoded_samples, qmode, bps, samples * num_chans);
-            MD5Update (&md5_global, (unsigned char *) decoded_samples, bps * samples * num_chans);
+            MD5_Update (&md5_global, (unsigned char *) decoded_samples, bps * samples * num_chans);
 
-            MD5Init (&md5_local);
-            MD5Update (&md5_local, (unsigned char *) decoded_samples, bps * samples * num_chans);
-            MD5Final (chunked_md5 + chunk_count * 16, &md5_local);
+            MD5_Init (&md5_local);
+            MD5_Update (&md5_local, (unsigned char *) decoded_samples, bps * samples * num_chans);
+            MD5_Final (chunked_md5 + chunk_count * 16, &md5_local);
 
             sample_count += samples;
             chunk_count++;
@@ -385,7 +385,7 @@ static int seeking_test (char *filename, uint32_t test_count)
         if (!test_index) {
             int file_has_md5 = WavpackGetMD5Sum (wpc, md5_stored), i;
 
-            MD5Final (md5_initial, &md5_global);
+            MD5_Final (md5_initial, &md5_global);
 
             for (i = 0; i < 16; ++i) {
                 sprintf (md5_string1 + (i * 2), "%02x", md5_stored [i]);
@@ -405,7 +405,7 @@ static int seeking_test (char *filename, uint32_t test_count)
         else {
             unsigned char md5_subsequent [16];
 
-            MD5Final (md5_subsequent, &md5_global);
+            MD5_Final (md5_subsequent, &md5_global);
 
             if (memcmp (md5_subsequent, md5_initial, sizeof (md5_stored))) {
                 printf ("seeking_test(): MD5 does not match MD5 read initially!\n");
@@ -464,9 +464,9 @@ static int seeking_test (char *filename, uint32_t test_count)
                 // if (frandom() < 0.0001)
                 //     decoded_samples [(int) floor (samples * frandom())] ^= 1;
 
-                MD5Init (&md5_local);
-                MD5Update (&md5_local, (unsigned char *) decoded_samples, bps * samples * num_chans);
-                MD5Final (md5_chunk, &md5_local);
+                MD5_Init (&md5_local);
+                MD5_Update (&md5_local, (unsigned char *) decoded_samples, bps * samples * num_chans);
+                MD5_Final (md5_chunk, &md5_local);
 
                 if (memcmp (chunked_md5 + current_chunk * 16, md5_chunk, sizeof (md5_chunk))) {
                     printf ("seeking_test(): seek+decode error at %lld!\n", (long long int) current_chunk * chunk_samples);
@@ -696,7 +696,7 @@ static int run_test (int wpconfig_flags, int test_flags, int bits, int num_chans
         strcat (mode_string, "hh");
 
     printf ("test %04d...", ++test_number); fflush (stdout);
-    MD5Init (&md5_context);
+    MD5_Init (&md5_context);
 
     noise_generator_init (&generators [0], 128.0);
     tone_generator_init (&generators [1], SAMPLE_RATE, 20, 200);
@@ -905,7 +905,7 @@ static int run_test (int wpconfig_flags, int test_flags, int bits, int num_chans
 
 	WavpackPackSamples (out_wpc, (int32_t *) destin, ENCODE_SAMPLES);
         store_samples (destin, (int32_t *) destin, 0, wpconfig.bytes_per_sample, ENCODE_SAMPLES * num_chans);
-        MD5Update (&md5_context, (unsigned char *) destin, wpconfig.bytes_per_sample * ENCODE_SAMPLES * num_chans);
+        MD5_Update (&md5_context, (unsigned char *) destin, wpconfig.bytes_per_sample * ENCODE_SAMPLES * num_chans);
 
         sequencing_angle += 2.0 * M_PI / SAMPLE_RATE / speed * ENCODE_SAMPLES;
         if (sequencing_angle > M_PI) sequencing_angle -= M_PI * 2.0;
@@ -931,7 +931,7 @@ static int run_test (int wpconfig_flags, int test_flags, int bits, int num_chans
     }
 
     WavpackFlushSamples (out_wpc);
-    MD5Final (md5_encoded, &md5_context);
+    MD5_Final (md5_encoded, &md5_context);
 
     if (wpconfig.flags & CONFIG_MD5_CHECKSUM) {
         WavpackStoreMD5Sum (out_wpc, md5_encoded);
@@ -1010,7 +1010,7 @@ static void *decode_thread (void *threadid)
         pthread_exit (NULL);
     }
 
-    MD5Init (&md5_context);
+    MD5_Init (&md5_context);
     num_chans = WavpackGetNumChannels (wpc);
     bps = WavpackGetBytesPerSample (wpc);
 
@@ -1028,11 +1028,11 @@ static void *decode_thread (void *threadid)
             break;
 
         store_samples (decoded_samples, decoded_samples, 0, bps, samples * num_chans);
-        MD5Update (&md5_context, (unsigned char *) decoded_samples, bps * samples * num_chans);
+        MD5_Update (&md5_context, (unsigned char *) decoded_samples, bps * samples * num_chans);
         wd->sample_count += samples;
     }
 
-    MD5Final (wd->md5_decoded, &md5_context);
+    MD5_Final (wd->md5_decoded, &md5_context);
     wd->num_errors = WavpackGetNumErrors (wpc);
     free (decoded_samples);
     WavpackCloseFile (wpc);
