@@ -104,13 +104,13 @@ static const unsigned char exp2_table [] = {
 
 ///////////////////////////// executable code ////////////////////////////////
 
-// Read the median log2 values from the specifed metadata structure, convert
+// Read the median log2 values from the specified metadata structure, convert
 // them back to 32-bit unsigned values and store them. If length is not
 // exactly correct then we flag and return an error.
 
 int read_entropy_vars (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    unsigned char *byteptr = wpmd->data;
+    unsigned char *byteptr = (unsigned char *)wpmd->data;
 
     if (wpmd->byte_length != ((wps->wphdr.flags & MONO_DATA) ? 6 : 12))
         return FALSE;
@@ -128,14 +128,14 @@ int read_entropy_vars (WavpackStream *wps, WavpackMetadata *wpmd)
     return TRUE;
 }
 
-// Read the hybrid related values from the specifed metadata structure, convert
+// Read the hybrid related values from the specified metadata structure, convert
 // them back to their internal formats and store them. The extended profile
 // stuff is not implemented yet, so return an error if we get more data than
 // we know what to do with.
 
 int read_hybrid_profile (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    unsigned char *byteptr = wpmd->data;
+    unsigned char *byteptr = (unsigned char *)wpmd->data;
     unsigned char *endptr = byteptr + wpmd->byte_length;
 
     if (wps->wphdr.flags & HYBRID_BITRATE) {
