@@ -130,7 +130,7 @@ void WavpackSetFileInformation (WavpackContext *wpc, char *file_extension, unsig
 //
 // The identities are provided in a NULL-terminated string (0x00 is not an allowed
 // channel ID). The Microsoft channels may be provided as well (and will be checked)
-// but it is really only neccessary to provide the "unknown" channels. Any truly
+// but it is really only necessary to provide the "unknown" channels. Any truly
 // unknown channels are indicated with a 0xFF.
 //
 // The channel IDs so far reserved are listed here:
@@ -190,11 +190,21 @@ int WavpackSetConfiguration (WavpackContext *wpc, WavpackConfig *config, uint32_
 
 int WavpackSetConfiguration64 (WavpackContext *wpc, WavpackConfig *config, int64_t total_samples, const unsigned char *chan_ids)
 {
-    uint32_t flags, bps = 0, shift = 0;
+    uint32_t flags, bps = 0;
     uint32_t chan_mask = config->channel_mask;
     int num_chans = config->num_channels;
     int i;
 
+    if (!config->sample_rate) {
+        strcpy (wpc->error_message, "sample rate cannot be zero!");
+        return FALSE;
+    }
+
+    if (!num_chans) {
+        strcpy (wpc->error_message, "channel count cannot be zero!");
+        return FALSE;
+    }
+
     wpc->stream_version = (config->flags & CONFIG_COMPATIBLE_WRITE) ? CUR_STREAM_VERS : MAX_STREAM_VERS;
 
     if ((config->qmode & QMODE_DSD_AUDIO) && config->bytes_per_sample == 1 && config->bits_per_sample == 8) {
@@ -248,18 +258,33 @@ int WavpackSetConfiguration64 (WavpackContext *wpc, WavpackConfig *config, int64
             break;
 
     flags |= i << SRATE_LSB;
-    flags |= shift << SHIFT_LSB;
 
     // all of this stuff only applies to PCM
 
     if (!(flags & DSD_FLAG)) {
         if (config->float_norm_exp) {
+            if (config->bytes_per_sample != 4 || config->bits_per_sample != 32) {
+                strcpy (wpc->error_message, "incorrect bits/bytes configuration for float data!");
+                return FALSE;
+            }
+
             wpc->config.float_norm_exp = config->float_norm_exp;
             wpc->config.flags |= CONFIG_FLOAT_DATA;
             flags |= FLOAT_DATA;
         }
-        else
-            shift = (config->bytes_per_sample * 8) - config->bits_per_sample;
+        else {
+            if (config->bytes_per_sample < 1 || config->bytes_per_sample > 4) {
+                strcpy (wpc->error_message, "invalid bytes per sample!");
+                return FALSE;
+            }
+
+            if (config->bits_per_sample < 1 || config->bits_per_sample > config->bytes_per_sample * 8) {
+                strcpy (wpc->error_message, "invalid bits per sample!");
+                return FALSE;
+            }
+
+            flags |= ((config->bytes_per_sample * 8) - config->bits_per_sample) << SHIFT_LSB;
+        }
 
         if (config->flags & CONFIG_HYBRID_FLAG) {
             flags |= HYBRID_FLAG | HYBRID_BITRATE | HYBRID_BALANCE;
@@ -838,24 +863,24 @@ static int create_riff_header (WavpackContext *wpc, int64_t total_samples, void
         wavhdr.GUID [13] = 0x71;
     }
 
-    strncpy (riffhdr.ckID, do_rf64 ? "RF64" : "RIFF", sizeof (riffhdr.ckID));
-    strncpy (riffhdr.formType, "WAVE", sizeof (riffhdr.formType));
+    memcpy (riffhdr.ckID, do_rf64 ? "RF64" : "RIFF", sizeof (riffhdr.ckID));
+    memcpy (riffhdr.formType, "WAVE", sizeof (riffhdr.formType));
     total_riff_bytes = sizeof (riffhdr) + wavhdrsize + sizeof (datahdr) + total_data_bytes + wpc->riff_trailer_bytes;
     if (do_rf64) total_riff_bytes += sizeof (ds64hdr) + sizeof (ds64_chunk);
     if (write_junk) total_riff_bytes += sizeof (junkchunk);
-    strncpy (fmthdr.ckID, "fmt ", sizeof (fmthdr.ckID));
-    strncpy (datahdr.ckID, "data", sizeof (datahdr.ckID));
+    memcpy (fmthdr.ckID, "fmt ", sizeof (fmthdr.ckID));
+    memcpy (datahdr.ckID, "data", sizeof (datahdr.ckID));
     fmthdr.ckSize = wavhdrsize;
 
     if (write_junk) {
         CLEAR (junkchunk);
-        strncpy (junkchunk.ckID, "junk", sizeof (junkchunk.ckID));
+        memcpy (junkchunk.ckID, "junk", sizeof (junkchunk.ckID));
         junkchunk.ckSize = sizeof (junkchunk) - 8;
         WavpackNativeToLittleEndian (&junkchunk, ChunkHeaderFormat);
     }
 
     if (do_rf64) {
-        strncpy (ds64hdr.ckID, "ds64", sizeof (ds64hdr.ckID));
+        memcpy (ds64hdr.ckID, "ds64", sizeof (ds64hdr.ckID));
         ds64hdr.ckSize = sizeof (ds64_chunk);
         CLEAR (ds64_chunk);
         ds64_chunk.riffSize64 = total_riff_bytes;
@@ -923,6 +948,7 @@ static int pack_streams (WavpackContext *wpc, uint32_t block_samples)
         max_blocksize += max_blocksize >> 2;    // otherwise 25% margin for everything else
 
     max_blocksize += wpc->metabytes + 1024;     // finally, add metadata & another 1K margin
+    max_blocksize += max_blocksize & 1;         // and make sure it's even so we detect overflow
 
     out2buff = (wpc->wvc_flag) ? malloc (max_blocksize) : NULL;
     out2end = out2buff + max_blocksize;
