@@ -17,7 +17,7 @@
 
 #include "wavpack_local.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 #define stricmp(x,y) _stricmp(x,y)
 #else
 #define stricmp strcasecmp
@@ -411,7 +411,7 @@ static int append_ape_tag_item (WavpackContext *wpc, const char *item, const cha
     int isize = (int) strlen (item);
 
     if (!m_tag->ape_tag_hdr.ID [0]) {
-        strncpy (m_tag->ape_tag_hdr.ID, "APETAGEX", sizeof (m_tag->ape_tag_hdr.ID));
+        memcpy (m_tag->ape_tag_hdr.ID, "APETAGEX", sizeof (m_tag->ape_tag_hdr.ID));
         m_tag->ape_tag_hdr.version = 2000;
         m_tag->ape_tag_hdr.length = sizeof (m_tag->ape_tag_hdr);
         m_tag->ape_tag_hdr.item_count = 0;
@@ -429,7 +429,7 @@ static int append_ape_tag_item (WavpackContext *wpc, const char *item, const cha
 
         m_tag->ape_tag_hdr.item_count++;
         m_tag->ape_tag_hdr.length += new_item_len;
-        p = m_tag->ape_tag_data = realloc (m_tag->ape_tag_data, m_tag->ape_tag_hdr.length);
+        p = m_tag->ape_tag_data = (unsigned char*)realloc (m_tag->ape_tag_data, m_tag->ape_tag_hdr.length);
         p += m_tag->ape_tag_hdr.length - sizeof (APE_Tag_Hdr) - new_item_len;
 
         *p++ = (unsigned char) vsize;
@@ -516,13 +516,13 @@ static int write_tag_reader (WavpackContext *wpc)
 
     // only write header if it's specified in the flags
 
-    if (m_tag->ape_tag_hdr.flags & APE_TAG_CONTAINS_HEADER)
+    if (tag_size && (m_tag->ape_tag_hdr.flags & APE_TAG_CONTAINS_HEADER))
         tag_size += sizeof (m_tag->ape_tag_hdr);
 
     result = !wpc->reader->set_pos_rel (wpc->wv_in, m_tag->tag_file_pos, SEEK_END);
 
-    if (result && tag_size < -m_tag->tag_file_pos) {
-        int nullcnt = -m_tag->tag_file_pos - tag_size;
+    if (result && tag_size < -m_tag->tag_file_pos && !wpc->reader->truncate_here) {
+        int nullcnt = (int) (-m_tag->tag_file_pos - tag_size);
         char zero [1] = { 0 };
 
         while (nullcnt--)
@@ -544,6 +544,9 @@ static int write_tag_reader (WavpackContext *wpc)
         WavpackLittleEndianToNative (&m_tag->ape_tag_hdr, APE_Tag_Hdr_Format);
     }
 
+    if (result && tag_size < -m_tag->tag_file_pos && wpc->reader->truncate_here)
+        result = !wpc->reader->truncate_here (wpc->wv_in);
+
     if (!result)
         strcpy (wpc->error_message, "can't write WavPack data, disk probably full!");
 
