@@ -20,7 +20,7 @@
 
 ///////////////////////////// executable code ////////////////////////////////
 
-// This function initialzes the main range-encoded data for DSD audio samples
+// This function initializes the main range-encoded data for DSD audio samples
 
 static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd);
 static int init_dsd_block_high (WavpackStream *wps, WavpackMetadata *wpmd);
@@ -34,7 +34,7 @@ int init_dsd_block (WavpackContext *wpc, WavpackMetadata *wpmd)
     if (wpmd->byte_length < 2)
         return FALSE;
 
-    wps->dsd.byteptr = wpmd->data;
+    wps->dsd.byteptr = (unsigned char *)wpmd->data;
     wps->dsd.endptr = wps->dsd.byteptr + wpmd->byte_length;
     wpc->dsd_multiplier = 1 << *wps->dsd.byteptr++;
     wps->dsd.mode = *wps->dsd.byteptr++;
@@ -64,7 +64,7 @@ int32_t unpack_dsd_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sampl
     // don't attempt to decode past the end of the block, but watch out for overflow!
 
     if (wps->sample_index + sample_count > GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples &&
-        GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples - wps->sample_index < sample_count)
+        (uint32_t) (GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples - wps->sample_index) < sample_count)
             sample_count = (uint32_t) (GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples - wps->sample_index);
 
     if (GET_BLOCK_INDEX (wps->wphdr) > wps->sample_index || wps->wphdr.block_samples < sample_count)
@@ -124,12 +124,11 @@ int32_t unpack_dsd_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sampl
 // #define DSD_BYTE_READY(low,high) (((low) >> 24) == ((high) >> 24))
 // #define DSD_BYTE_READY(low,high) (!(((low) ^ (high)) >> 24))
 #define DSD_BYTE_READY(low,high) (!(((low) ^ (high)) & 0xff000000))
-#define MAX_HISTORY_BITS    5
 
 static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
 {
-    unsigned char history_bits, max_probability;
-    int total_summed_probabilities = 0, i;
+    unsigned char history_bits, max_probability, *lb_ptr;
+    int total_summed_probabilities = 0, bi, i;
 
     if (wps->dsd.byteptr == wps->dsd.endptr)
         return FALSE;
@@ -141,10 +140,12 @@ static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
 
     wps->dsd.history_bins = 1 << history_bits;
 
-    wps->dsd.value_lookup = malloc (sizeof (*wps->dsd.value_lookup) * wps->dsd.history_bins);
+    free_dsd_tables (wps);
+    lb_ptr = wps->dsd.lookup_buffer = (unsigned char *)malloc (wps->dsd.history_bins * MAX_BYTES_PER_BIN);
+    wps->dsd.value_lookup = (unsigned char **)malloc (sizeof (*wps->dsd.value_lookup) * wps->dsd.history_bins);
     memset (wps->dsd.value_lookup, 0, sizeof (*wps->dsd.value_lookup) * wps->dsd.history_bins);
-    wps->dsd.summed_probabilities = malloc (sizeof (*wps->dsd.summed_probabilities) * wps->dsd.history_bins);
-    wps->dsd.probabilities = malloc (sizeof (*wps->dsd.probabilities) * wps->dsd.history_bins);
+    wps->dsd.summed_probabilities = (int16_t (*)[256])malloc (sizeof (*wps->dsd.summed_probabilities) * wps->dsd.history_bins);
+    wps->dsd.probabilities = (unsigned char (*)[256])malloc (sizeof (*wps->dsd.probabilities) * wps->dsd.history_bins);
 
     max_probability = *wps->dsd.byteptr++;
 
@@ -177,27 +178,28 @@ static int init_dsd_block_fast (WavpackStream *wps, WavpackMetadata *wpmd)
     else
         return FALSE;
 
-    for (wps->dsd.p0 = 0; wps->dsd.p0 < wps->dsd.history_bins; ++wps->dsd.p0) {
+    for (bi = 0; bi < wps->dsd.history_bins; ++bi) {
         int32_t sum_values;
-        unsigned char *vp;
 
         for (sum_values = i = 0; i < 256; ++i)
-            wps->dsd.summed_probabilities [wps->dsd.p0] [i] = sum_values += wps->dsd.probabilities [wps->dsd.p0] [i];
+            wps->dsd.summed_probabilities [bi] [i] = sum_values += wps->dsd.probabilities [bi] [i];
 
         if (sum_values) {
-            total_summed_probabilities += sum_values;
-            vp = wps->dsd.value_lookup [wps->dsd.p0] = malloc (sum_values);
+            if ((total_summed_probabilities += sum_values) > wps->dsd.history_bins * MAX_BYTES_PER_BIN)
+                return FALSE;
+
+            wps->dsd.value_lookup [bi] = lb_ptr;
 
             for (i = 0; i < 256; i++) {
-                int c = wps->dsd.probabilities [wps->dsd.p0] [i];
+                int c = wps->dsd.probabilities [bi] [i];
 
                 while (c--)
-                    *vp++ = i;
+                    *lb_ptr++ = i;
             }
         }
     }
 
-    if (wps->dsd.endptr - wps->dsd.byteptr < 4 || total_summed_probabilities > wps->dsd.history_bins * 1280)
+    if (wps->dsd.endptr - wps->dsd.byteptr < 4 || total_summed_probabilities > wps->dsd.history_bins * MAX_BYTES_PER_BIN)
         return FALSE;
 
     for (i = 4; i--;)
@@ -316,7 +318,9 @@ static int init_dsd_block_high (WavpackStream *wps, WavpackMetadata *wpmd)
     if (rate_s != RATE_S)
         return FALSE;
 
-    wps->dsd.ptable = malloc (PTABLE_BINS * sizeof (*wps->dsd.ptable));
+    if (!wps->dsd.ptable)
+        wps->dsd.ptable = (int32_t *)malloc (PTABLE_BINS * sizeof (*wps->dsd.ptable));
+
     init_ptable (wps->dsd.ptable, rate_i, rate_s);
 
     for (channel = 0; channel < ((flags & MONO_DATA) ? 1 : 2); ++channel) {
@@ -440,9 +444,11 @@ static int decode_high (WavpackStream *wps, int32_t *output, int sample_count)
 
 /*------------------------------------------------------------------------------------------------------------------------*/
 
+#if 0
+
 // 80 term DSD decimation filter
 // < 1 dB down at 20 kHz
-// > 108 dB stopband attenuation
+// > 108 dB stopband attenuation (fs/16)
 
 static const int32_t decm_filter [] = {
     4, 17, 56, 147, 336, 693, 1320, 2359,
@@ -457,7 +463,28 @@ static const int32_t decm_filter [] = {
     2359, 1320, 693, 336, 147, 56, 17, 4,
 };
 
-#define NUM_FILTER_TERMS ((int)(sizeof (decm_filter) / sizeof (decm_filter [0])))
+#define NUM_FILTER_TERMS 80
+
+#else
+
+// 56 term decimation filter
+// < 0.5 dB down at 20 kHz
+// > 100 dB stopband attenuation (fs/12)
+
+static const int32_t decm_filter [] = {
+    4, 17, 56, 147, 336, 692, 1315, 2337,
+    3926, 6281, 9631, 14216, 20275, 28021, 37619, 49155,
+    62616, 77870, 94649, 112551, 131049, 149507, 167220, 183448,
+    197472, 208636, 216402, 220385, 220385, 216402, 208636, 197472,
+    183448, 167220, 149507, 131049, 112551, 94649, 77870, 62616,
+    49155, 37619, 28021, 20275, 14216, 9631, 6281, 3926,
+    2337, 1315, 692, 336, 147, 56, 17, 4,
+};
+
+#define NUM_FILTER_TERMS 56
+
+#endif
+
 #define HISTORY_BYTES ((NUM_FILTER_TERMS+7)/8)
 
 typedef struct {
@@ -472,7 +499,7 @@ typedef struct {
 
 void *decimate_dsd_init (int num_channels)
 {
-    DecimationContext *context = malloc (sizeof (DecimationContext));
+    DecimationContext *context = (DecimationContext *)malloc (sizeof (DecimationContext));
     double filter_sum = 0, filter_scale;
     int skipped_terms, i, j;
 
@@ -481,7 +508,7 @@ void *decimate_dsd_init (int num_channels)
 
     memset (context, 0, sizeof (*context));
     context->num_channels = num_channels;
-    context->chans = malloc (num_channels * sizeof (DecimationChannel));
+    context->chans = (DecimationChannel *)malloc (num_channels * sizeof (DecimationChannel));
 
     if (!context->chans) {
         free (context);
@@ -540,6 +567,7 @@ void decimate_dsd_run (void *decimate_context, int32_t *samples, int num_samples
         DecimationChannel *sp = context->chans + chan;
         int sum = 0;
 
+#if (HISTORY_BYTES == 10)
         sum += context->conv_tables [0] [sp->delay [0] = sp->delay [1]];
         sum += context->conv_tables [1] [sp->delay [1] = sp->delay [2]];
         sum += context->conv_tables [2] [sp->delay [2] = sp->delay [3]];
@@ -550,6 +578,23 @@ void decimate_dsd_run (void *decimate_context, int32_t *samples, int num_samples
         sum += context->conv_tables [7] [sp->delay [7] = sp->delay [8]];
         sum += context->conv_tables [8] [sp->delay [8] = sp->delay [9]];
         sum += context->conv_tables [9] [sp->delay [9] = *samples];
+#elif (HISTORY_BYTES == 7)
+        sum += context->conv_tables [0] [sp->delay [0] = sp->delay [1]];
+        sum += context->conv_tables [1] [sp->delay [1] = sp->delay [2]];
+        sum += context->conv_tables [2] [sp->delay [2] = sp->delay [3]];
+        sum += context->conv_tables [3] [sp->delay [3] = sp->delay [4]];
+        sum += context->conv_tables [4] [sp->delay [4] = sp->delay [5]];
+        sum += context->conv_tables [5] [sp->delay [5] = sp->delay [6]];
+        sum += context->conv_tables [6] [sp->delay [6] = *samples];
+#else
+        int i;
+
+        for (i = 0; i < HISTORY_BYTES-1; ++i)
+            sum += context->conv_tables [i] [sp->delay [i] = sp->delay [i+1]];
+
+        sum += context->conv_tables [i] [sp->delay [i] = *samples];
+#endif
+
         *samples++ = sum >> 4;
 
         if (++chan == context->num_channels) {
