@@ -37,11 +37,11 @@
 uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t samples)
 {
     WavpackStream *wps = wpc->streams ? wpc->streams [wpc->current_stream = 0] : NULL;
+    int num_channels = wpc->config.num_channels, file_done = FALSE;
     uint32_t bcount, samples_unpacked = 0, samples_to_unpack;
-    int num_channels = wpc->config.num_channels;
-    int file_done = FALSE;
+    int32_t *bptr = buffer;
 
-#ifndef VER4_ONLY
+#ifdef ENABLE_LEGACY
     if (wpc->stream3)
         return unpack_samples3 (wpc, buffer, samples);
 #endif
@@ -53,9 +53,9 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
         // to free up the streams and read the next block
 
         if (!wps->wphdr.block_samples || !(wps->wphdr.flags & INITIAL_BLOCK) ||
-            wps->sample_index >= wps->wphdr.block_index + wps->wphdr.block_samples) {
+            wps->sample_index >= GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples) {
 
-                uint32_t nexthdrpos;
+                int64_t nexthdrpos;
 
                 if (wpc->wrapper_bytes >= MAX_WRAPPER_BYTES)
                     break;
@@ -69,14 +69,9 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
 
                 wpc->filepos = nexthdrpos + bcount;
 
-                if (wpc->open_flags & OPEN_STREAMING)
-                    wps->wphdr.block_index = wps->sample_index = 0;
-                else
-                    wps->wphdr.block_index -= wpc->initial_index;
-
                 // allocate the memory for the entire raw block and read it in
 
-                wps->blockbuff = malloc (wps->wphdr.ckSize + 8);
+                wps->blockbuff = (unsigned char *)malloc (wps->wphdr.ckSize + 8);
 
                 if (!wps->blockbuff)
                     break;
@@ -91,11 +86,26 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
                         break;
                 }
 
+                // render corrupt blocks harmless
+                if (!WavpackVerifySingleBlock (wps->blockbuff, !(wpc->open_flags & OPEN_NO_CHECKSUM))) {
+                    wps->wphdr.ckSize = sizeof (WavpackHeader) - 8;
+                    wps->wphdr.block_samples = 0;
+                    memcpy (wps->blockbuff, &wps->wphdr, 32);
+                }
+
+                // potentially adjusting block_index must be done AFTER verifying block
+
+                if (wpc->open_flags & OPEN_STREAMING)
+                    SET_BLOCK_INDEX (wps->wphdr, wps->sample_index = 0);
+                else
+                    SET_BLOCK_INDEX (wps->wphdr, GET_BLOCK_INDEX (wps->wphdr) - wpc->initial_index);
+
+                memcpy (wps->blockbuff, &wps->wphdr, 32);
                 wps->init_done = FALSE;     // we have not yet called unpack_init() for this block
 
                 // if this block has audio, but not the sample index we were expecting, flag an error
 
-                if (wps->wphdr.block_samples && wps->sample_index != wps->wphdr.block_index)
+                if (wps->wphdr.block_samples && wps->sample_index != GET_BLOCK_INDEX (wps->wphdr))
                     wpc->crc_errors++;
 
                 // if this block has audio, and we're in hybrid lossless mode, read the matching wvc block
@@ -118,16 +128,18 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
         // to loop back and read the next block
 
         if (!wps->wphdr.block_samples || !(wps->wphdr.flags & INITIAL_BLOCK) ||
-            wps->sample_index >= wps->wphdr.block_index + wps->wphdr.block_samples)
+            wps->sample_index >= GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples)
                 continue;
 
         // There seems to be some missing data, like a block was corrupted or something.
         // If it's not too much data, just fill in with silence here and loop back.
 
-        if (wps->sample_index < wps->wphdr.block_index) {
-            samples_to_unpack = wps->wphdr.block_index - wps->sample_index;
+        if (wps->sample_index < GET_BLOCK_INDEX (wps->wphdr)) {
+            int32_t zvalue = (wps->wphdr.flags & DSD_FLAG) ? 0x55 : 0;
 
-            if (samples_to_unpack > 262144) {
+            samples_to_unpack = (uint32_t) (GET_BLOCK_INDEX (wps->wphdr) - wps->sample_index);
+
+            if (!samples_to_unpack || samples_to_unpack > 262144) {
                 strcpy (wpc->error_message, "discontinuity found, aborting file!");
                 wps->wphdr.block_samples = 0;
                 wps->wphdr.ckSize = 24;
@@ -141,13 +153,10 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
             samples_unpacked += samples_to_unpack;
             samples -= samples_to_unpack;
 
-            if (wpc->reduced_channels)
-                samples_to_unpack *= wpc->reduced_channels;
-            else
-                samples_to_unpack *= num_channels;
+            samples_to_unpack *= (wpc->reduced_channels ? wpc->reduced_channels : num_channels);
 
             while (samples_to_unpack--)
-                *buffer++ = 0;
+                *bptr++ = zvalue;
 
             continue;
         }
@@ -155,7 +164,7 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
         // calculate number of samples to process from this block, then initialize the decoder for
         // this block if we haven't already
 
-        samples_to_unpack = wps->wphdr.block_index + wps->wphdr.block_samples - wps->sample_index;
+        samples_to_unpack = (uint32_t) (GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples - wps->sample_index);
 
         if (samples_to_unpack > samples)
             samples_to_unpack = samples;
@@ -169,7 +178,7 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
         // to stereo), then enter this conditional block...otherwise we just unpack the samples directly
 
         if (!wpc->reduced_channels && !(wps->wphdr.flags & FINAL_BLOCK)) {
-            int32_t *temp_buffer = malloc (samples_to_unpack * 8), *src, *dst;
+            int32_t *temp_buffer = (int32_t *)malloc (samples_to_unpack * 8), *src, *dst;
             int offset = 0;     // offset to next channel in sequence (0 to num_channels - 1)
             uint32_t samcnt;
 
@@ -186,12 +195,12 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
                 // if the stream has not been allocated and corresponding block read, do that here...
 
                 if (wpc->current_stream == wpc->num_streams) {
-                    wpc->streams = realloc (wpc->streams, (wpc->num_streams + 1) * sizeof (wpc->streams [0]));
+                    wpc->streams = (WavpackStream **)realloc (wpc->streams, (wpc->num_streams + 1) * sizeof (wpc->streams [0]));
 
                     if (!wpc->streams)
 			break;
 
-                    wps = wpc->streams [wpc->num_streams++] = malloc (sizeof (WavpackStream));
+                    wps = wpc->streams [wpc->num_streams++] = (WavpackStream *)malloc (sizeof (WavpackStream));
 
                     if (!wps)
 			break;
@@ -206,12 +215,7 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
                         break;
                     }
 
-                    if (wpc->open_flags & OPEN_STREAMING)
-                        wps->wphdr.block_index = wps->sample_index = 0;
-                    else
-                        wps->wphdr.block_index -= wpc->initial_index;
-
-                    wps->blockbuff = malloc (wps->wphdr.ckSize + 8);
+                    wps->blockbuff = (unsigned char *)malloc (wps->wphdr.ckSize + 8);
 
                     if (!wps->blockbuff)
 		        break;
@@ -226,6 +230,22 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
                             break;
                     }
 
+                    // render corrupt blocks harmless
+                    if (!WavpackVerifySingleBlock (wps->blockbuff, !(wpc->open_flags & OPEN_NO_CHECKSUM))) {
+                        wps->wphdr.ckSize = sizeof (WavpackHeader) - 8;
+                        wps->wphdr.block_samples = 0;
+                        memcpy (wps->blockbuff, &wps->wphdr, 32);
+                    }
+
+                    // potentially adjusting block_index must be done AFTER verifying block
+
+                    if (wpc->open_flags & OPEN_STREAMING)
+                        SET_BLOCK_INDEX (wps->wphdr, wps->sample_index = 0);
+                    else
+                        SET_BLOCK_INDEX (wps->wphdr, GET_BLOCK_INDEX (wps->wphdr) - wpc->initial_index);
+
+                    memcpy (wps->blockbuff, &wps->wphdr, 32);
+
                     // if this block has audio, and we're in hybrid lossless mode, read the matching wvc block
 
                     if (wpc->wvc_flag)
@@ -243,9 +263,15 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
 
                 // unpack the correct number of samples (either mono or stereo) into the temp buffer
 
-                unpack_samples (wpc, src = temp_buffer, samples_to_unpack);
+#ifdef ENABLE_DSD
+                if (wps->wphdr.flags & DSD_FLAG)
+                    unpack_dsd_samples (wpc, src = temp_buffer, samples_to_unpack);
+                else
+#endif
+                    unpack_samples (wpc, src = temp_buffer, samples_to_unpack);
+
                 samcnt = samples_to_unpack;
-                dst = buffer + offset;
+                dst = bptr + offset;
 
                 // if the block is mono, copy the samples from the single channel into the destination
                 // using num_channels as the stride
@@ -296,7 +322,16 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
             // if we didn't get all the channels we expected, mute the buffer and flag an error
 
             if (offset != num_channels) {
-                memset (buffer, 0, samples_to_unpack * num_channels * 4);
+                if (wps->wphdr.flags & DSD_FLAG) {
+                    int samples_to_zero = samples_to_unpack * num_channels;
+                    int32_t *zptr = bptr;
+
+                    while (samples_to_zero--)
+                        *zptr++ = 0x55;
+                }
+                else
+                    memset (bptr, 0, samples_to_unpack * num_channels * 4);
+
                 wpc->crc_errors++;
             }
 
@@ -309,12 +344,16 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
         // catch the error situation where we have only one channel but run into a stereo block
         // (this avoids overwriting the caller's buffer)
         else if (!(wps->wphdr.flags & MONO_FLAG) && (num_channels == 1 || wpc->reduced_channels == 1)) {
-            memset (buffer, 0, samples_to_unpack * sizeof (*buffer));
+            memset (bptr, 0, samples_to_unpack * sizeof (*bptr));
             wps->sample_index += samples_to_unpack;
             wpc->crc_errors++;
         }
+#ifdef ENABLE_DSD
+        else if (wps->wphdr.flags & DSD_FLAG)
+            unpack_dsd_samples (wpc, bptr, samples_to_unpack);
+#endif
         else
-            unpack_samples (wpc, buffer, samples_to_unpack);
+            unpack_samples (wpc, bptr, samples_to_unpack);
 
         if (file_done) {
             strcpy (wpc->error_message, "can't read all of last block!");
@@ -322,9 +361,9 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
         }
 
         if (wpc->reduced_channels)
-            buffer += samples_to_unpack * wpc->reduced_channels;
+            bptr += samples_to_unpack * wpc->reduced_channels;
         else
-            buffer += samples_to_unpack * num_channels;
+            bptr += samples_to_unpack * num_channels;
 
         samples_unpacked += samples_to_unpack;
         samples -= samples_to_unpack;
@@ -332,8 +371,18 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
         // if we just finished a block, check for a calculated crc error
         // (and back up the streams a little if possible in case we passed a header)
 
-        if (wps->sample_index == wps->wphdr.block_index + wps->wphdr.block_samples) {
+        if (wps->sample_index == GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples) {
             if (check_crc_error (wpc)) {
+                int32_t *zptr = bptr, zvalue = (wps->wphdr.flags & DSD_FLAG) ? 0x55 : 0;
+                uint32_t samples_to_zero = wps->wphdr.block_samples;
+
+                if (samples_to_zero > samples_to_unpack)
+                    samples_to_zero = samples_to_unpack;
+
+                samples_to_zero *= (wpc->reduced_channels ? wpc->reduced_channels : num_channels);
+
+                while (samples_to_zero--)
+                    *--zptr = zvalue;
 
                 if (wps->blockbuff && wpc->reader->can_seek (wpc->wv_in)) {
                     int32_t rseek = ((WavpackHeader *) wps->blockbuff)->ckSize / 3;
@@ -349,9 +398,14 @@ uint32_t WavpackUnpackSamples (WavpackContext *wpc, int32_t *buffer, uint32_t sa
             }
         }
 
-        if (wpc->total_samples != (uint32_t) -1 && wps->sample_index == wpc->total_samples)
+        if (wpc->total_samples != -1 && wps->sample_index == wpc->total_samples)
             break;
     }
 
+#ifdef ENABLE_DSD
+    if (wpc->decimation_context)
+        decimate_dsd_run (wpc->decimation_context, buffer, samples_unpacked);
+#endif
+
     return samples_unpacked;
 }
